/***************************************************************************
                  
                          configfile.cpp 
                          class for config-file handling
                             -------------------
    begin                : Mon June 29 2003
    copyright            : (C) 2003 by Thomas Unterthiner
    email                : thomas_unterthiner@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "configfile.h"



ConfigFile::ConfigFile(const string& filename)
/***************************************************************************
 * ctor, opens the file and reads in the settings
 ***************************************************************************/
 : m_file( filename.c_str(), ios::in | ios::out | ios::app )
{
	readSettings();
}




/***************************************************************************
 * the following overloaded functions return a setting
 *	PARAMETER
 *		setting_name: the name of the setting, as it is written in the 
 *		              configfile
 *		destination: setting will be stored in this value
 *	RETURN
 *		TRUE if setting could be returned, otherwise FALSE, and of course
 *		the searched setting using the destination-pointer recieved as
 *		parameter, 
 ***************************************************************************/
bool ConfigFile::getSetting(const string& setting_name, long* destination) const
{
	map<string, string>::const_iterator it = m_settings.find(setting_name);
	if(it!=m_settings.end())
	{
			istringstream tmp(it->second);
			tmp >> *destination;
			return true;
	}
	return false;	
}

bool ConfigFile::getSetting(const string& setting_name, double* destination) const
{
	map<string, string>::const_iterator it = m_settings.find(setting_name);
	if ( it != m_settings.end() )
	{
		istringstream tmp(it->second);
		tmp >> *destination;
		return true;
	}
	return false;	
}

bool ConfigFile::getSetting(const string& setting_name, char* destination) const
{
	map<string, string>::const_iterator it = m_settings.find(setting_name);
	if(it!=m_settings.end())
	{
		strcpy(destination, it->second.c_str() );
		return true;
	}
	else
	{
		destination = "";
		return false;
	}
}


/***************************************************************************
 * the following overloaded functions write an entry into the config-file
 *	PARAMETER
 *		key: the name of the setting, as it is written in the configfile
 *		value: value of the key
 *	RETURN
 *		TRUE if setting could be found, FALSE if it had to be created
 ***************************************************************************/
bool ConfigFile::set(const string& key, long value)
{
	ostringstream tmp;
    tmp << value;
	return setEntry( key, tmp.str() );
}

bool ConfigFile::set(const string& key, double value)
{
	ostringstream tmp;
    tmp << value;
	return setEntry( key, tmp.str() );
}

bool ConfigFile::set(const string& key, const string& value)
{
	return setEntry( key, value );
}


// -----------------------------------
//  		PRIVATE MEMBERS
// -----------------------------------


void ConfigFile::readSettings()
/***************************************************************************
 * ctor, opens the file and reads in the settings
 ***************************************************************************/
{
	string tmp_str;
	string key;
	string value;
	int border_pos;
	int end_pos;

	do
	{
		std::getline(m_file, tmp_str);
		
		// jump over comment-lines
		if (tmp_str[0] == COMMENT_SIGN )
		{
			continue;
		}

		// find and read the setting
		border_pos = tmp_str.find_first_of(ASSIGN_SIGN);
		end_pos = tmp_str.find_last_of(END_SIGN);
		key = tmp_str.substr(0, border_pos);
		value = tmp_str.substr(border_pos+1, end_pos);

		// store setting
		m_settings[key] = value;
//		m_settings.insert( m_settings::value_type(key, value) );
	}while ( !m_file.eof() );
}


streampos ConfigFile::searchEntry(const string& text, streampos start_pos)
/***************************************************************************
 * searches an Entry in the configfile
 * 	PARAMETER
 * 		text: text to search for
 * 		start_pos: begin starting here (default = 0)
 ***************************************************************************/
{
	// reset filestatus
	m_file.clear();
	m_file.seekg(start_pos);

	long key_pos = 0;
	string::size_type pos;
	string tmp;

	do
	{
		getline(m_file, tmp);
        pos = tmp.find(text);
		if ( pos != string::npos ) 	// text found!
		{
			key_pos += pos;
			return key_pos;
		}
		else
			key_pos += tmp.length()+1;	// +1 for '\n';

	}while ( !m_file.eof() );

	// could not find that thing....
	return -1;
}

void ConfigFile::changeValue(const string& key,
                             const string& value,
							 streampos entry_pos)
/***************************************************************************
 * changes the value of an existing entry...
 * I found no faster way then reading all data that could possibly be
 * overwritten and rewrite it to the file after changing the entry
 * 	PARAMETER
 * 		key: the key
 * 		value: the value
 * 		entry_pos: the position within the file where the line with the
 *                 key begins
 ***************************************************************************/
{
    // read in the rest of the file
	streampos start_reading = searchEntry(END_SIGN, entry_pos) + 1;
	m_file.seekg(0, ios::end);
	unsigned long buffer_size = m_file.tellg() - start_reading;
	char* buffer = new char[buffer_size];
	m_file.seekp(start_reading);
	m_file.read(buffer, buffer_size);  // binary reading is faster
    
	// now change the entry
	m_file.seekp(entry_pos);
	m_file << key << ASSIGN_SIGN << value << END_SIGN << '\n';
	
	// rewrite the rest of the file
	start_reading = searchEntry(END_SIGN, entry_pos) + 1;
	m_file.write(buffer, buffer_size);
	return;
}


void ConfigFile::insertEntry(const string& key, const string& value)
/***************************************************************************
 * inserts a new entry at the end of the file
 * 	PARAMETER
 * 		key: the key
 * 		value: the value
 ***************************************************************************/
{
	m_file.clear();
	m_file.seekp(0, ios::end);
	m_file << key << ASSIGN_SIGN << value << END_SIGN << '\n';
	return;
}

